# Alligator

_Aligátor sežral Pelikána... R.I.P._

Pirátská SMS brána.

[![code style: Black](https://img.shields.io/badge/code%20style-Black-000000)](https://github.com/psf/black)
[![powered by: Django](https://img.shields.io/badge/powered%20by-Django-0C4B33)](https://www.djangoproject.com)

## Pod pokličkou

### Struktura projektu

    .
    ├── alligator       = Django projekt s konfigurací
    └── users           = app s custom user modelem a SSO, apod.

## Deployment

### Konfigurace

Je třeba nastavit environment proměnné:

| proměnná | default | popis |
| --- | --- | --- |
| `DATABASE_URL` | | DSN k databázi (např. `postgres://user:pass@localhost:5342/alligator`) |
| `OIDC_RP_REALM_URL` | | OpenID server realm URL (např. `http://localhost:8080/realms/master/`) |
| `OIDC_RP_CLIENT_ID` | | OpenID Client ID |
| `OIDC_RP_CLIENT_SECRET` | | OpenID Client Secret |

V produkci musí být navíc nastaveno:

| proměnná | default | popis |
| --- | --- | --- |
| `DJANGO_SECRET_KEY` | | tajný šifrovací klíč |
| `DJANGO_ALLOWED_HOSTS` | | allowed hosts (více hodnot odděleno čárkami) |
| `DJANGO_SETTINGS_MODULE` | `alligator.settings.prod` | produkční settings |

Různé:

| proměnná | default | popis |
| --- | --- | --- |
| `SENTRY_DSN` | | pokud je zadáno, pády se reportují do Sentry |

### Management commands

Přes CRON je třeba na pozadí spouštět Django `manage.py` commandy:

* `clearsessions` - maže expirované sessions (denně až týdně)

## Vývoj

Pro vývoj je definován pomocný `Makefile` pro časté akce. Pro nápovědu zavolej:

    $ make help

### Lokální instalace a spuštění

#### Vytvoření virtualenv pro instalaci závislostí

Vytvoř virtualenv:

    $ make venv

Vytvoří virtualenv ve složce `.venv`. Předpokládá že výchozí `python` v terminálu
je Python 3. Pokud tomu tak není, použijte třeba [Pyenv](https://github.com/pyenv/pyenv)
pro instalaci více verzí Pythonu bez rizika rozbití systému.

#### Aktivace virtualenvu

Před prací na projektu je třeba aktivovat virtualenv. To bohužel nejde dělat
pomocí nástroje `make`. Je třeba zavolat příkaz:

    $ source .venv/bin/activate

Můžete asi na to vytvořit alias pro shell. Do `~/.bash_profile` nebo `~/.zshrc`
nebo jiného konfiguračního souboru dle vašeho shellu přidejte:

    alias senv='source .venv/bin/activate'

A pak můžete virtualenv aktivovat pouze jednoduchým voláním:

    $ senv

Pro sofistikovanější řešení, které vám aktivuje virtualenv při změně adresáře na
adresář s projektem, slouží nástroj [direnv](https://direnv.net/).

Deaktivace virtualenvu se dělá příkazem:

    $ deactivate

#### Instalace závislostí

V aktivovaném virtualenvu spusťte:

    $ make install

To nainstaluje Pythonní závislosti pro vývoj projektu na lokále.

#### Nastavení environment proměnných

Environment proměnné (viz konfigurace výše) se načítají ze souboru `.env`, který
může vypadat takto:

    DATABASE_URL=postgres://db:db@localhost:5432/alligator
    OIDC_RP_REALM_URL=http://localhost:8080/realms/master/
    OIDC_RP_CLIENT_ID=alligator
    OIDC_RP_CLIENT_SECRET=abcd

Pro lokální vývoj obsahují settings tyto výchozí hodnoty:

    DEBUG = True
    ALLOWED_HOSTS = ["*"]

### Management projektu

#### Migrace databáze

Aplikuj migrace databáze:

    $ make migrate

Při změně modelů vygeneruj migrace pomocí:

    $ make migrations

#### Spuštění development serveru

Django development server na portu `8017` se spustí příkazem:

    $ make run

Poté můžete otevřít web na adrese [http://localhost:8017](http://localhost:8017)

#### Django shell

Django shell používající `shell_plus` z Django extensions spustíte:

    $ make shell

### Testy

Používá se testovací framework [pytest](https://pytest.org). Spuštění testů:

    $ pytest

Případně přes `make`, ale bez možnosti parametrizovat spuštění testů:

    $ make test

Coverage report:

    $ make coverage

### Code quality

K formátování kódu se používá [black](https://github.com/psf/black). Doporučujeme
ho nainstalovat do vašeho editoru kódu, aby soubory přeformátoval po uložení.

Přeformátování kódu nástrojem `black` je součástí `pre-commit` hooks (viz níže).

Součástí `pre-commit` hooků je také automatické seřazení importů v Pythonních
souborech nástrojem [isort](https://github.com/timothycrosley/isort/).

### Pre-commit hooky

Použivá se [pre-commit](https://pre-commit.com/) framework pro management git
pre-commit hooks.

Máte-li pre-commit framework [nainstalovaný](https://pre-commit.com/#installation)
spusttě příkaz:

    $ make install-hooks

Ten naisntaluje hooky pro projekt. A poté při každém commitu dojde k požadovaným
akcím na změněných souborech.

Ručně se dají hooky na všechny soubory spustit příkazem:

    $ make hooks

## Upgrade závislostí

K upgrade se používají [pip-tools](https://github.com/jazzband/pip-tools) (`pip install pip-tools`):

    $ cd requirements/
    $ pip-compile -U base.in
    $ pip-compile -U prod.in
    $ pip-compile -U dev.in

Tím se vygenerují `base.txt`, `prod.txt` a `dev.txt`.

Nebo to stejné lze provést příkazem:

    $ make upgrade
